function changeName(type, char_obj, str)
    tostring(str)

    -- Only if new name is given
    if str ~= '' then
        local game_langs = {}
        local save_table = nil

        local lines = ek_explode(str, '<br/>') -- split the new name into lines
        local name_id = char_obj:getName() -- get the identifier name of the object

--[[
        All the new names (and the default ones of those objects that got a new name)
        have to be stored in global variables, because otherwise they won't be saved
        in savegames.
]]

        -- 1. Renaming object is character
        if type == 'char' then
            if char_names == nil then char_names = {} end
            if char_orig_names == nil then char_orig_names = {} end

            -- Save the default names in a global table, using the identifier name as identifier
            if char_orig_names[name_id] == nil then
                char_orig_names[name_id] = char_obj.Name.TextLanguages
            end

            -- New names will be stored in this global table (passed by reference)
            save_table = char_names
        else
            -- 2. Renaming object is item
            if char_obj.IsItem then
                if item_names == nil then item_names = {} end
                if item_orig_names == nil then item_orig_names = {} end

                -- Save the default names in a global table, using the identifier name as identifier
                if item_orig_names[name_id] == nil then
                    item_orig_names[name_id] = char_obj.Name.TextLanguages
                end

                -- New names will be stored in this global table (passed by reference)
                save_table = item_names
            else
                -- 3. Renaming object is scene object
                local scene_found = false
                local scene_name = ''

                -- Find the name of the scene the object belongs to
                for k, scene in ipairs(Scenes) do
                    for i, scene_obj in ipairs(scene.Objects) do
                        if scene_obj:getId().id == char_obj:getId().id then
                            scene_name = scene:getName()

                            scene_found = true
                            break
                        end
                    end

                    if scene_found then break end
                end

                if scene_name ~= '' then
                    if obj_names == nil then obj_names = {} end
                    if obj_names[scene_name] == nil then obj_names[scene_name] = {} end
                    if obj_orig_names == nil then obj_orig_names = {} end
                    if obj_orig_names[scene_name] == nil then obj_orig_names[scene_name] = {} end

                    -- Save the default names in a global table, using the scene name and the identifier name as identifiers
                    if obj_orig_names[scene_name][name_id] == nil then
                        obj_orig_names[scene_name][name_id] = char_obj.Name.TextLanguages
                    end

                    -- New names will be stored in this global table (passed by reference)
                    save_table = obj_names[scene_name]
                end
            end
        end

        if save_table ~= nil then
            -- Get the current names of the object
            save_table[name_id] = char_obj.Name.TextLanguages

            -- Build a table of the game languages
            for k, lang in ipairs(game.Languages) do
                game_langs[lang:getName()] = lang:getId().id
            end

            -- Loop through the lines of the new name string
            for i = 1, #lines do
                -- If not empty
                if ek_trim(lines[i]) ~= '' then
                    -- Find first equals sign in the line
                    pos = lines[i]:find('=')
                 
                    if pos ~= nil and pos > 1 then
                        -- If equals sign found, check if the preceding string equals one of the game languages:
                        -- if it does, save the string following the equals sign as the new name for this language
                        for k = 1, #save_table[name_id] do
                            if save_table[name_id][k].language == game_langs[ ek_trim(lines[i]:sub(1, pos - 1)) ] then
                                save_table[name_id][k].text = ek_trim(lines[i]:sub(pos + 1))
                            end
                        end
                    -- If no equals sign is found and if the new name string consists of one line only,
                    -- set this string as the name for all languages
                    elseif #lines == 1 then
                        for k = 1, #save_table[name_id] do
                            save_table[name_id][k].text = ek_trim(lines[i])
                        end
                    end
                end
            end

            -- Set the new table as the names of the object
            char_obj.Name.TextLanguages = save_table[name_id]
        end
    end
end


--[[
    The reload function gets all entries from the global variables and assigns them
    back to the objects. It has to be executed after loading a savegame.
]]

function reloadResetNames(switch)
    if switch == 1 then
        -- If "reset" option selected, loop through the three global tables with default names
        -- and assign them to the objects; delete the global table of new names
        if char_orig_names ~= nil then
            for k, v in pairs(char_orig_names) do
                Characters[k].Name.TextLanguages = char_orig_names[k]
                char_names[k] = nil
            end
        end
        if item_orig_names ~= nil then
            for k, v in pairs(item_orig_names) do
                game.Items[k].Name.TextLanguages = item_orig_names[k]
                item_names[k] = nil
            end
        end
        if obj_orig_names ~= nil then
            for k, v in pairs(obj_orig_names) do
                for n, w in pairs(v) do
                    Scenes[k].Objects[n].Name.TextLanguages = v[n]
                    obj_names[n] = nil
                end
                
                obj_names[k] = nil
            end
        end
    else
        -- If "reload" option selected, loop through the three global tables of new names
        -- and assign them to the objects
        if char_names ~= nil then
            for k, v in pairs(char_names) do
                Characters[k].Name.TextLanguages = char_names[k]
            end
        end
        if item_names ~= nil then
            for k, v in pairs(item_names) do
                game.Items[k].Name.TextLanguages = item_names[k]
            end
        end
        if obj_names ~= nil then
            for k, v in pairs(obj_names) do
                for n, w in pairs(v) do
                    Scenes[k].Objects[n].Name.TextLanguages = v[n]
                end
            end
        end
    end
end



-- Helper functions

function ek_explode(str, delimiter)
    result = {}
    
    for match in (str .. delimiter):gmatch("(.-)" .. delimiter) do
        table.insert(result, match)
    end

    return result
end

function ek_trim(str)
    return (str:gsub("^%s*(.-)%s*$", "%1"))
end



-- Register action parts

registerActionPart('CHANGE_CHARACTER_NAME', function(activeAction, actionPart)
    changeName('char', actionPart.Link, actionPart.String)
    return false
end)

registerActionPart('CHANGE_OBJECT_NAME', function(activeAction, actionPart)
    changeName('obj', actionPart.Link, actionPart.String)
    return false
end)

registerActionPart('RELOAD_RESET_NAMES', function(activeAction, actionPart)
    reloadResetNames(actionPart.Int)
    return false
end)